#include <stdint.h>
#include <stdio.h>
#include <time.h>

#include "common.h"
#include "unpack.h"

void print_buf_ofs (unsigned char *buf, size_t size, size_t ofs)
{
    size_t pos=0;
    unsigned starting_offset=0;
    unsigned i;

    while (size-pos)
    {
        size_t wpn;
        if ((size-pos)>16)
            wpn=16;
        else
            wpn=size-pos;

        printf ("0x%08zx: ", starting_offset + pos + ofs);
        
        for (i=0; i<wpn; i++)
            printf ("%02X%c", buf[pos+i], (i==7) ? '-' : ' ');

        if (wpn<16)
            for (i=0; i<16-wpn; i++)
                printf ("   ");

        printf ("\"");

        for (i=0; i<wpn; i++)
            printf ("%c", isprint (buf[pos+i]) ? buf[pos+i] : '.');

        if (wpn<16)
            for (i=0; i<16-wpn; i++)
                printf (" ");

        printf ("\"\n");

        pos+=wpn;
    };
};

struct lic
{
        uint32_t sig;      // 4 bytes. must be 0xc0d1f1ed
        uint32_t SN;       // 4 bytes
        uint64_t exp_date; // 8 bytes
        uint32_t features; // 4 bytes
        char name[28]; // incl. terminating zero
};

void main(int argc, char* argv[])
{
        printf ("Challenge/keygenme #88 by Dennis Yurichev\n");
        if (argc!=2)
        {
                printf ("Usage: %s <license code>\n", argv[0]);
                return;
        };
        char buf[256];
        unpack_lic_code_hex(buf, argv[1]);
        char* buf_ptr=buf;

        while (*buf_ptr=='0')
                buf_ptr++;

        struct lic l;
        memset (&l, 0, 48);
        unsigned char* ptr=(unsigned char*)&l;
        //printf ("%p\n", ptr);
        for (int i=0; i<48; i++)
        {
                if (buf_ptr[i]==0)
                        break;
                char buf2[3];
                buf2[0]=buf_ptr[i*2];
                buf2[1]=buf_ptr[i*2+1];
                buf2[2]=0;
                unsigned int tmp;
                //printf (buf2);
                sscanf (buf2, "%x", &tmp);
                //printf ("%d %02x\n", i, tmp);
                ptr[i]=tmp;
        };
        //print_buf_ofs(&l, 48, 0);
        //printf ("%p\n", &l);
        struct tm* tmp;
        tmp=localtime(&l.exp_date);
        //printf ("Sig: %x\n", l.sig);
        if (l.sig!=0xc0d1f1ed)
        {
                printf ("Invalid license code\n");
                return;
        };
        printf ("Valid license code:\n");
        printf ("Serial number: %d\n", l.SN);
        printf ("Expiration date: %s", asctime (tmp));
        //printf ("Features: %d\n", l.features);
        printf ("Name: %s\n", l.name);
};

