#include <time.h>
#include <assert.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include "common.h"
#include "pack.h"
#include "unpack.h"

void print_buf_ofs (unsigned char *buf, size_t size, size_t ofs)
{
    size_t pos=0;
    unsigned starting_offset=0;
    unsigned i;

    while (size-pos)
    {
        size_t wpn;
        if ((size-pos)>16)
            wpn=16;
        else
            wpn=size-pos;

        printf ("0x%08zx: ", starting_offset + pos + ofs);
        
        for (i=0; i<wpn; i++)
            printf ("%02X%c", buf[pos+i], (i==7) ? '-' : ' ');

        if (wpn<16)
            for (i=0; i<16-wpn; i++)
                printf ("   ");

        printf ("\"");

        for (i=0; i<wpn; i++)
            printf ("%c", isprint (buf[pos+i]) ? buf[pos+i] : '.');

        if (wpn<16)
            for (i=0; i<16-wpn; i++)
                printf (" ");

        printf ("\"\n");

        pos+=wpn;
    };
};

void print_without_leading_zeroes(char *s)
{
        while (*s=='0')
                s++;
        printf ("%s\n", s);
};

void test(char *s)
{
        char tmp[202];
        char tmp2[202];
        pack_lic_code(tmp, s);
        print_without_leading_zeroes (tmp);
        unpack_lic_code_hex(tmp2, tmp);
        print_without_leading_zeroes (tmp2);
};

struct lic
{
        uint32_t sig;      // 4 bytes. must be 0xc0d1f1ed
        uint32_t SN;       // 4 bytes
        uint64_t exp_date; // 8 bytes
        uint32_t features; // 4 bytes
        char name[28]; // incl. terminating zero
};

void gen_license(char* out, int exp_year, int exp_mon, int exp_day, int SN, char* name)
{
        struct lic l;
        struct tm t;

        //printf ("%s\n", asctime(&t));
        //printf ("%d\n", mktime(&t));

        memset(&t, 0, sizeof(struct tm));
        t.tm_mday=exp_day;
        t.tm_mon=exp_mon;
        t.tm_year=exp_year-1900;
        memset (&l, 0, sizeof(struct lic));
        l.sig=0xc0d1f1ed;
        l.SN=SN;
        l.exp_date=mktime(&t);
        l.features=1;
        strcpy (l.name, name);

        unsigned char* ptr=(unsigned char*)&l;

        char buf[202];
        memset (buf, 0, 202);

	const char *hex="0123456789abcdef";
        for (int i=0; i<48; i++)
	{
		buf[i*2]=hex[(ptr[i]>>4)&0xf];
		buf[i*2+1]=hex[ptr[i]&0xf];
	};
        //printf (buf);
        pack_lic_code(out, buf);
};

void main()
{
        assert(sizeof(struct lic)==48);

        char buf[202];

        gen_license(buf, 2022, 0, 1, 1234, "Dennis Yurichev");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2024, 5, 6, 1111, "Test User");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2030, 1, 1, 1234567890, "John Doe");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2040, 10, 10, 1, "Jane Doe");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2020, 1, 1, 999, "Dummy");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2010, 1, 1, 999999, "SpongeBob SquarePants");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2022, 11, 30, 87654321, "Squidward");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2022, 11, 30, 1, "Mr. Krabs");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2022, 11, 30, 123, "Alf");
        print_without_leading_zeroes(buf);
        gen_license(buf, 2022, 11, 30, 1234, "Rocko");
        print_without_leading_zeroes(buf);

        exit(0);

        test("ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"); // 48 bytes
        test("f00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001"); // 48 bytes
        test("100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001"); // 48 bytes
        test("1");
        test("2");
};
